# frozen_string_literal: true

module Deeds
  class Journal

    attr_reader :hours_per_work_day, :work_days_per_week

    def initialize(file:, hours_per_work_day: 8, work_days_per_week: 5)
      @file = Pathname.new(file)
      @hours_per_work_day = hours_per_work_day
      @work_days_per_week = work_days_per_week
    end

    def dump
      entries.map(&:dump).join "\n"
    end

    def entries
      all_entries.reject(&:error)
    end

    def invalid_entries
      all_entries.select(&:error)
    end

    def void_entries
      all_entries.reject(&:minutes?)
    end

    def minutes
      BigDecimal.new entries.sum(&:minutes)
    end

    def hours
      minutes / 60
    end

    def full_time_days
      hours / hours_per_work_day
    end

    def full_time_weeks
      full_time_days / work_days_per_week
    end

    def full_time_months
      full_time_weeks / (52 / 12)
    end

    def days_count
      entries_per_day.keys.count
    end

    def average_minutes_per_day
      return 0 if minutes.zero?
      minutes / days_count
    end

    def average_hours_per_day
      return 0 if average_minutes_per_day.zero?
      average_minutes_per_day / 60
    end

    def first_monday
      @first_monday ||= entries.sort_by(&:mjd).detect(&:monday?).try!(:date)
    end

    def last_sunday
      @last_sunday ||= entries.sort_by(&:mjd).reverse.detect(&:sunday?).try!(:date)
    end

    def days_with_more_than_one_entry
      entries_per_day.values.select { |entries| entries.count > 1 }.count
    end

    private

    attr_reader :file

    def all_entries
      @all_entries ||= all_entries!
    end

    def all_entries!
      result = []
      file.each_line.with_index do |line, index|
        next if line.blank?
        next if line.strip.start_with?('#')
        result << Journals::Entry.new(input: line.chomp, line: index + 1)
      end
      result
    end

    def entries_per_day
      @entries_per_day ||= entries_per_day!
    end

    def entries_per_day!
      result = {}
      entries.each do |entry|
        next if entry.date.nil?
        result[entry.date.mjd] ||= []
        result[entry.date.mjd] << entry
      end
      result
    end

  end
end
